# Example WAIF service implemented using the WAIFService module
# You are encouraged to look at the module source code if you want
# details on its behaviour or attributes.
# Most common features are shown in this example. More advanced stuff
# will be shown in group sessions upon request.


from WAIFService import Handler, Listener
from logging import DEBUG


class ws_example(Listener):
    """Subclassing the Listener class expands our ws_example class and
    gives us lots of cool functionality. See literature on object
    orientation in Python for details on what happens here."""

    def __init__(self):
        
        self.name = "ws_example"
        
        Listener.__init__(self,
                          self.name,
                          port=8090
                          )

        # After this operation, we get two useful attributes that are
        # mandatory in the handler constructor just to make sure the
        # programmer knows about them..
        # self.jobQueue and self.logger
        # these can also be overridden if desired, like if you want to
        # log to a HTTP server instead of a local file.
        
        self.handler = Handler ( self,
                                 self.name,
                                 self.jobQueue,
                                 self.logger
                                 )

    def main(self):
        self.logger.info('MAIN: Ready to jumpstart %s WAIFService!'%self.name)

        # making sure we are logging in debug level so we get ALL logentries.
        if self.logger.getEffectiveLevel() > DEBUG:
            self.logger.setLevel(DEBUG)

        try:
            # registering event handlers
            self.handler.registerFunction(self.myLittleHandler)
            self.handler.registerFunction(self.myOtherHandler)
            
            self.handler.start()  # starting handler thread
            self.run()            # starting this thread


        except KeyboardInterrupt:
            self.logger.info("Got Ctrl-C; Stopping %s server" % self.name)
                        
        except:
            self.logger.exception("Critical failure: SHUTTING DOWN!")
            print "Critical failure, inspect log %s.log" % self.name



###########  IMPLEMENTING MAIN FUNCTIONALITY OF WS_EXAMPLE.PY #######

    def myLittleHandler(self, event):
        waifID, subID, handle, args = event
        
        profile = self.users[waifID][subID]
        
        self.logger.info( "%s got an event: %s" % (self.name, event))

        newmsg = {}
        if args and type(args) == type({}):
            newmsg = args
        
        newmsg['data'] = ['not much', 'info in', 'this event']
        newmsg['DataType'] = 'list'

        self.logger.debug("Trying to send event: %s" % newmsg )

        self.handler.push( profile['taddr'],
                           waifID,
                           profile['remoteID'],
                           newmsg )


    def myOtherHandler(self, event):

        self.logger.info("myOtherHandler got an event, but I don't care")
        
        return


# if this is a standalone application; run main function:
if __name__ == '__main__':

    exSrv = ws_example()
    exSrv.main()
